import click

from estela_cli.login import login
from estela_cli.utils import (
    get_estela_settings,
    validate_key_value_format,
    set_tag_format,
    validate_positive,
)


SHORT_HELP = "Create a new job"


@click.command(short_help=SHORT_HELP)
@click.argument("sid", required=True)
@click.argument("pid", required=False)
@click.option(
    "--arg",
    "-a",
    multiple=True,
    type=click.UNPROCESSED,
    callback=validate_key_value_format,
    help="Set spider job argument NAME=VALUE (may be repeated)",
)
@click.option(
    "--env",
    "-e",
    multiple=True,
    type=click.UNPROCESSED,
    callback=validate_key_value_format,
    help="Set spider job environment variable NAME=VALUE (may be repeated)",
)
@click.option(
    "--tag",
    "-t",
    multiple=True,
    type=click.UNPROCESSED,
    callback=set_tag_format,
    help="Set spider job tag (may have multiple)",
)
@click.option(
    "--day",
    "-d",
    type=click.INT,
    callback=validate_positive,
    help="Set spider job data expiry days",
)
def estela_command(sid, pid, arg, env, tag, day):
    """Create a new job

    \b
    SID is the spider's sid
    PID is the project's pid (active project by default)
    """

    estela_client = login()
    if pid is None:
        try:
            estela_settings = get_estela_settings()
            pid = estela_settings["project"]["pid"]
        except:
            raise click.ClickException(
                "No active project in the current directory. Please specify the PID."
            )
    try:
        response = estela_client.create_spider_job(pid, sid, arg, env, tag, day)
        click.echo("job/{} created.".format(response["name"]))
    except Exception as ex:
        raise click.ClickException("Cannot create the job for given SID and PID.")



# Auth templates

ESTELA_AUTH_NAME = ".estela.yaml"

ESTELA_AUTH = """\
token: $estela_token
host: $estela_host
"""


# Init project templates

ESTELA_DIR = ".estela"

DATA_DIR = "project_data"

DOCKER_APP_DIR = "/usr/src/app"

DOCKER_DEFAULT_PYTHON_VERSION = "3.9"

DOCKER_DEFAULT_REQUIREMENTS = "requirements.txt"

DOCKER_DEFAULT_ENTRYPOINT = "git+https://github.com/bitmakerla/estela-entrypoint.git"

DOCKER_REQUESTS_ENTRYPOINT = "git+https://github.com/bitmakerla/estela-requests-entrypoint"

DOCKERFILE_NAME = "Dockerfile-estela"

DOCKERFILE = """\
FROM python:$python_version

# must be in base image
RUN pip install $entrypoint
RUN mkdir -p {app_dir}
WORKDIR {app_dir}
COPY . {app_dir}

RUN pip install --no-cache-dir -r $requirements_path
RUN mkdir /fifo-data
""".format(
    app_dir=DOCKER_APP_DIR
)

ESTELA_YAML_NAME = "estela.yaml"

ESTELA_YAML = """\
project:
  pid: "$project_pid"
  python: "$python_version"
  requirements: "$requirements_path"
  entrypoint: "$entrypoint"
deploy:
  ignore: ["$project_data_path"]
"""

# General templates

OK_EMOJI = "\U00002705"
BAD_EMOJI = "\U0000274C"
CLOCK_EMOJI = "\U000023F3"



import click

from estela_cli.login import login


SHORT_HELP = "Delete an existing project"


@click.command(short_help=SHORT_HELP)
@click.argument("pid", required=True)
def estela_command(pid):
    """Delete a project

    PID is the project's pid
    """

    estela_client = login()
    try:
        estela_client.delete_project(pid)
        click.echo("project/{} deleted.".format(pid))
    except Exception as ex:
        raise click.ClickException(
            "The project does not exist, or you do not have permission to perform this action."
        )



import click

from estela_cli.login import login
from estela_cli.utils import get_estela_settings, validate_positive

SHORT_HELP = "Update a job"


@click.command(short_help=SHORT_HELP)
@click.argument("jid", required=True)
@click.argument("sid", required=True)
@click.argument("pid", required=False)
@click.option(
    "--persistent",
    is_flag=True,
    default=None,
    help="Set job data as persistent.",
)
@click.option(
    "--day",
    "-d",
    type=click.INT,
    callback=validate_positive,
    help="Set spider job data expiry days.",
)
def estela_command(jid, sid, pid, day, persistent):
    """Update a job

    \b
    SID is the spider's sid
    PID is the project's pid (active project by default)
    JID is the job's jid
    """

    estela_client = login()
    if pid is None:
        try:
            estela_settings = get_estela_settings()
            pid = estela_settings["project"]["pid"]
        except:
            raise click.ClickException(
                "No active project in the current directory. Please specify the PID."
            )

    if all([option is None for option in [day, persistent]]):
        raise click.ClickException(
            "No update option was provided to update the job. Please specify at least one."
        )

    try:
        response = estela_client.update_spider_job(pid, sid, jid, day, persistent)
        click.echo(f"job/spider-job-{jid}-{pid} updated.")
    except Exception as ex:
        raise click.ClickException(
            "Some values are invalid or you do not have permission to perform this action."
        )



import click

from estela_cli.login import login


SHORT_HELP = "Create a new project"


@click.command(short_help=SHORT_HELP)
@click.argument("name", required=True)
def estela_command(name):
    """Create a new project

    NAME is the project's name
    """

    estela_client = login()
    try:
        response = estela_client.create_project(name)
        click.echo("project/{} created.".format(name))
        click.echo(
            f"Hint: Use 'estela init {response['pid']}' to initialize a Scrapy project.\n"
            f"Hint: Use 'estela init {response['pid']} -p requests' to initialize a Requests project."
        )
    except Exception as ex:
        raise click.ClickException(str(ex))



from datetime import date, timedelta

import requests

from estela_cli import __version__


class EstelaSimpleClient:
    token = None

    def __init__(self, host, username=None, password=None, token=None):
        self.host = host
        self.api_base = "{}/api".format(self.host)
        if token is None:
            data = {
                "username": username,
                "password": password,
            }
            response = self.post("auth/login", data)
            self.check_status(response, 200, "non_field_errors")
            self.token = response.json()["key"]
        else:
            self.token = token
            response = self.get("projects")
            self.check_status(response, 200)

    def url_for(self, endpoint):
        return "{}/{}".format(self.api_base, endpoint)

    def get_default_headers(self):
        headers = {"User-Agent": f"estela-cli/{__version__}"}
        if self.token:
            headers["Authorization"] = "Token {}".format(self.token)
        return headers

    def post(self, endpoint, data=None, params=None, files=None):
        if files is None:
            files = {}
        if params is None:
            params = {}
        if data is None:
            data = {}
        headers = self.get_default_headers()
        return requests.post(
            self.url_for(endpoint),
            json=data,
            headers=headers,
            params=params,
            files=files,
        )

    def get(self, endpoint, params=None, paginated=False):
        if params is None:
            params = {}
        headers = self.get_default_headers()
        response = requests.get(self.url_for(endpoint), headers=headers, params=params)
        if paginated:
            self.check_status(response, 200)
            response = response.json()
            next_page = response["next"]
            content = response["results"]
            while next_page:
                response = requests.get(
                    self.url_for(endpoint), headers=headers, params=params
                )
                self.check_status(response, 200)
                response = response.json()
                content += response["results"]
                next_page = response["next"]
            return content
        return response

    def put(self, endpoint, data=None, params=None):
        if params is None:
            params = {}
        if data is None:
            data = {}
        headers = self.get_default_headers()
        return requests.put(
            self.url_for(endpoint), data=data, headers=headers, params=params
        )

    def delete(self, endpoint, data=None, params=None):
        if params is None:
            params = {}
        if data is None:
            data = {}
        headers = self.get_default_headers()
        return requests.delete(
            self.url_for(endpoint), headers=headers, data=data, params=params
        )

    def check_status(self, response, status_code, error_field="detail"):
        if response.status_code != status_code:
            response_json = response.json()
            if error_field in response_json:
                raise Exception(response_json[error_field])
            else:
                raise Exception(str(response_json))


class EstelaClient(EstelaSimpleClient):
    def get_projects(self):
        endpoint = "projects"
        response = self.get(endpoint, paginated=True)
        return response

    def get_project(self, pid):
        endpoint = "projects/{}".format(pid)
        response = self.get(endpoint)
        self.check_status(response, 200)
        return response.json()

    def create_project(self, name):
        endpoint = "projects"
        data = {"name": name}
        response = self.post(endpoint, data=data)
        self.check_status(response, 201)
        return response.json()
    
    def update_project(self, pid, **kwargs):
        endpoint = "projects/{}".format(pid)
        data = {
            **kwargs,
        }
        response = self.put(endpoint, data=data)
        self.check_status(response, 200)
        return response.json()

    def delete_project(self, pid):
        endpoint = "projects/{}".format(pid)
        response = self.delete(endpoint)
        self.check_status(response, 204)

    def get_spiders(self, pid):
        endpoint = "projects/{}/spiders".format(pid)
        response = self.get(endpoint, paginated=True)
        return response

    def get_spider(self, pid, sid):
        endpoint = "projects/{}/spiders/{}".format(pid, sid)
        response = self.get(endpoint)
        self.check_status(response, 200)
        return response.json()

    def upload_project(self, pid, project_zip):
        endpoint = "projects/{}/deploys".format(pid)
        files = {"project_zip": project_zip}
        response = self.post(endpoint, files=files)
        self.check_status(response, 201)
        return response.json()

    def get_spider_cronjobs(self, pid, sid):
        endpoint = "projects/{}/spiders/{}/cronjobs".format(pid, sid)
        response = self.get(endpoint, paginated=True)
        return response

    def get_spider_cronjobs_with_tag(self, pid, sid, tag):
        endpoint = "projects/{}/spiders/{}/cronjobs?tag={}".format(pid, sid, tag)
        response = self.get(endpoint, paginated=True)
        return response

    def get_spider_cronjob(self, pid, sid, cjid):
        endpoint = "projects/{}/spiders/{}/cronjobs/{}".format(pid, sid, cjid)
        response = self.get(endpoint)
        self.check_status(response, 200)
        return response.json()

    def get_spider_jobs(self, pid, sid):
        endpoint = "projects/{}/spiders/{}/jobs".format(pid, sid)
        response = self.get(endpoint, paginated=True)
        return response

    def get_spider_jobs_with_tag(self, pid, sid, tag):
        endpoint = "projects/{}/spiders/{}/jobs?tag={}".format(pid, sid, tag)
        response = self.get(endpoint, paginated=True)
        return response

    def get_spider_job(self, pid, sid, jid):
        endpoint = "projects/{}/spiders/{}/jobs/{}".format(pid, sid, jid)
        response = self.get(endpoint)
        self.check_status(response, 200)
        return response.json()

    def get_spider_job_data(self, pid, sid, jid, datatype, last_chunk=None):
        endpoint = "projects/{}/spiders/{}/jobs/{}/data?type={}".format(
            pid, sid, jid, datatype
        )
        if last_chunk:
            endpoint += "&current_chunk={}".format(last_chunk)
        response = self.get(endpoint)
        self.check_status(response, 200)
        return response.json()

    def create_spider_job(self, pid, sid, args=[], env_vars=[], tags=[], day=None):
        endpoint = "projects/{}/spiders/{}/jobs".format(pid, sid)
        data = {
            "args": args,
            "env_vars": env_vars,
            "tags": tags,
            "data_status": "PENDING" if day else "PERSISTENT",
        }
        if day:
            data["data_expiry_days"] = f"{date.today() + timedelta(days=day)}"

        response = self.post(endpoint, data=data)
        self.check_status(response, 201)
        return response.json()

    def stop_spider_job(self, pid, sid, jid):
        endpoint = "projects/{}/spiders/{}/jobs/{}/stop".format(pid, sid, jid)
        response = self.put(endpoint)
        self.check_status(response, 200)
        return response.json()

    def create_spider_cronjob(
        self, pid, sid, schedule="", args=[], env_vars=[], tags=[], day=None
    ):
        endpoint = "projects/{}/spiders/{}/cronjobs".format(pid, sid)
        data = {
            "schedule": schedule,
            "cargs": args,
            "cenv_vars": env_vars,
            "ctags": tags,
            "data_status": "PENDING" if day else "PERSISTENT",
        }
        if day:
            data["data_expiry_days"] = f"0/{day}"

        response = self.post(endpoint, data=data)
        self.check_status(response, 201)
        return response.json()

    def update_spider_job(self, pid, sid, jid, day, persistent):
        endpoint = "projects/{}/spiders/{}/jobs/{}".format(pid, sid, jid)
        data = {}
        if persistent:
            data["data_status"] = "PERSISTENT"
        elif day and day >= 1:
            data["data_status"] = "PENDING"
            data["data_expiry_days"] = day

        response = self.put(endpoint, data=data)
        self.check_status(response, 200)
        return response.json()

    def update_spider_cronjob(self, pid, sid, cjid, status, schedule, day, persistent):
        endpoint = "projects/{}/spiders/{}/cronjobs/{}".format(pid, sid, cjid)
        data = {
            "status": status,
            "schedule": schedule,
        }
        if persistent:
            data["data_status"] = "PERSISTENT"
        if day and day >= 1:
            data["data_status"] = "PENDING"
            data["data_expiry_days"] = day

        response = self.put(endpoint, data=data)
        self.check_status(response, 200)
        return response.json()



import click
import importlib


@click.group(short_help="Delete a resource")
def estela_command():
    pass


commands = [
    "project",
]

for command in commands:
    module = importlib.import_module("estela_cli.delete.{}".format(command))
    estela_command.add_command(module.estela_command, command)



import click

from tabulate import tabulate
from estela_cli.login import login


SHORT_HELP = "List the current user's projects"


@click.command(short_help=SHORT_HELP)
def estela_command():
    estela_client = login()
    projects = estela_client.get_projects()
    projects = [[project["name"], project["pid"]] for project in projects]

    headers = ["NAME", "PID"]
    click.echo(tabulate(projects, headers, numalign="left", tablefmt="plain"))



import time

import click

from estela_cli.login import login
from estela_cli.templates import BAD_EMOJI, CLOCK_EMOJI, OK_EMOJI
from estela_cli.utils import get_estela_settings, save_chunk_data, save_data

SHORT_HELP = "Retrieve data from a job"
ALLOWED_DATA_TYPES = ["items", "requests", "logs", "stats"]
ALLOWED_DATA_FORMATS = ["json", "csv", "tsv"]


@click.command(short_help=SHORT_HELP)
@click.argument("jid", required=True)
@click.argument("sid", required=True)
@click.argument("pid", required=False)
@click.option(
    "-t",
    "--datatype",
    type=click.Choice(ALLOWED_DATA_TYPES, case_sensitive=False),
    default="items",
)
@click.option(
    "-f",
    "--format",
    type=click.Choice(ALLOWED_DATA_FORMATS, case_sensitive=False),
    default="json",
)
def estela_command(
    jid,
    sid,
    pid,
    datatype,
    format,
):
    """Retrieve data from a given job

    \b
    SID is the spider's sid
    PID is the project's pid (active project by default)
    JID is the job's id
    DATATYPE is the type of data to retrieve
    FORMAT is the format to retrieve data
    """

    estela_client = login()
    if pid is None:
        try:
            estela_settings = get_estela_settings()
            pid = estela_settings["project"]["pid"]
        except:
            raise click.ClickException(
                "No active project in the current directory. Please specify the PID."
            )

    tmp_filename = ".{}-{}-{}-{}-tmp".format(jid, pid, datatype, time.time())
    filename = "{}-{}-{}.{}".format(jid, pid, datatype, format)
    try:
        response = estela_client.get_spider_job_data(pid, sid, jid, datatype)
        with click.progressbar(
            length=int(response["count"]),
            label="{} Downloading job data".format(CLOCK_EMOJI),
            show_eta=True,
            show_percent=True,
            show_pos=True,
        ) as progress_bar:
            next_chunk = None
            while True:
                response = estela_client.get_spider_job_data(
                    pid, sid, jid, datatype, next_chunk
                )
                chunk = response.get("results")
                save_chunk_data(tmp_filename, chunk)
                progress_bar.update(len(chunk))
                next_chunk = response.get("next_chunk")
                if next_chunk is None:
                    break
        click.echo("{} Data downloaded succesfully.".format(OK_EMOJI))
    except:
        raise click.ClickException(
            "{} Could not download the job data".format(BAD_EMOJI)
        )

    try:
        save_data(filename, tmp_filename, format)
        click.echo("{} Data saved succesfully.".format(OK_EMOJI))
    except:
        raise click.ClickException("{} Could not save the job data".format(BAD_EMOJI))



import os
import click

from string import Template
from zipfile import ZipFile, ZIP_DEFLATED
from estela_cli.utils import (
    get_project_path,
    get_estela_settings,
    _in,
    get_estela_dockerfile_path,
)
from estela_cli.login import login
from estela_cli.templates import (
    OK_EMOJI,
    ESTELA_DIR,
    ESTELA_YAML_NAME,
    DOCKERFILE,
    DOCKERFILE_NAME,
)


SHORT_HELP = "Deploy Scrapy project to estela API"


def zip_project(pid, project_path, estela_settings):
    relroot = os.path.abspath(os.path.join(project_path, os.pardir))
    archives_to_ignore = estela_settings["deploy"]["ignore"]
    with ZipFile("{}.zip".format(pid), "w", ZIP_DEFLATED) as zip:
        for root, dirs, files in os.walk(project_path):
            # ignoring dir with data from jobs
            rel_root = root.replace("{}/".format(project_path), "")
            if _in(rel_root, archives_to_ignore):
                continue
            # add directory (needed for empty dirs)
            zip.write(root, os.path.relpath(root, relroot))
            for file in files:
                filename = os.path.join(root, file)
                arcname = os.path.join(os.path.relpath(root, relroot), file)
                zip.write(filename, arcname)


def update_dockerfile(requirements_path, python_version, entrypoint):
    dockerfile_path = get_estela_dockerfile_path()

    project_path = get_project_path()
    requirements_local_path = os.path.join(project_path, requirements_path)
    if not os.path.exists(requirements_local_path):
        raise click.ClickException("The requirements file does not exist.")

    template = Template(DOCKERFILE)
    values = {
        "python_version": python_version,
        "requirements_path": requirements_path,
        "entrypoint": entrypoint,
    }
    result = template.substitute(values)
    with open(dockerfile_path, "r") as dock:
        if result == dock.read():
            click.echo(
                "{}/{} not changes to update.".format(ESTELA_DIR, DOCKERFILE_NAME)
            )
            return

    with open(dockerfile_path, "w+") as dockerfile:
        dockerfile.write(result)
        click.echo("{}/{} updated successfully.".format(ESTELA_DIR, DOCKERFILE_NAME))


@click.command(short_help=SHORT_HELP)
def estela_command():
    estela_client = login()
    estela_settings = get_estela_settings()
    project_path = get_project_path()
    p_settings = estela_settings["project"]
    pid = p_settings["pid"]

    try:
        estela_client.get_project(pid)
    except:
        raise click.ClickException(
            "Invalid project at {}/{}.".format(ESTELA_DIR, ESTELA_YAML_NAME)
        )

    update_dockerfile(
        p_settings["requirements"],
        p_settings["python"],
        p_settings["entrypoint"],
    )

    zip_project(pid, project_path, estela_settings)

    response = {}
    try:
        response = estela_client.upload_project(pid, open("{}.zip".format(pid), "rb"))
    except:
        os.remove("{}.zip".format(pid))
        raise click.ClickException("A problem occurred while uploading the project.")

    click.echo(
        "{} Project uploaded successfully. Deploy {} underway.".format(
            OK_EMOJI, response.get("did")
        )
    )
    os.remove("{}.zip".format(pid))



import click

from tabulate import tabulate
from estela_cli.login import login
from estela_cli.utils import (
    get_estela_settings,
    format_time,
    format_key_value_pairs,
    format_tags,
)

SHORT_HELP = "List cronjobs of a given spider"


@click.command(short_help=SHORT_HELP)
@click.argument("sid", required=True)
@click.argument("pid", required=False)
@click.option(
    "--tag",
    "-t",
    type=click.UNPROCESSED,
    help="Filter cronjobs by tag",
)
def estela_command(sid, pid, tag):
    """List cronjobs of a given spider

    \b
    SID is the spider's sid
    PID is the project's pid (active project by default)
    TAG is the tag used to filter cronjobs
    """

    estela_client = login()

    if pid is None:
        try:
            estela_settings = get_estela_settings()
            pid = estela_settings["project"]["pid"]
        except:
            raise click.ClickException(
                "No active project in the current directory. Please specify the PID."
            )

    try:
        estela_client.get_spider(pid, sid)
    except Exception as ex:
        raise click.ClickException(
            "The spider does not exist, or you do not have permission to perform this action."
        )

    cronjobs = []
    if tag:
        cronjobs = estela_client.get_spider_cronjobs_with_tag(pid, sid, tag)
    else:
        cronjobs = estela_client.get_spider_cronjobs(pid, sid)

    cronjobs = [
        [
            cronjob["cjid"],
            cronjob["status"],
            cronjob["schedule"],
            format_tags(cronjob["ctags"]),
            format_key_value_pairs(cronjob["cargs"]),
            format_key_value_pairs(cronjob["cenv_vars"]),
        ]
        for cronjob in cronjobs
    ]

    headers = ["CJID", "STATUS", "SCHEDULE", "TAGS", "ARGS", "ENV VARS"]
    click.echo(tabulate(cronjobs, headers, numalign="left", tablefmt="plain"))



import click

from tabulate import tabulate
from estela_cli.login import login
from estela_cli.utils import get_estela_settings


SHORT_HELP = "List a project's spiders"


@click.command(short_help=SHORT_HELP)
@click.argument("pid", required=False)
def estela_command(pid):
    """List spiders of a given project

    PID is the project's pid (active project by default)
    """

    estela_client = login()

    if pid is None:
        try:
            estela_settings = get_estela_settings()
            pid = estela_settings["project"]["pid"]
        except:
            raise click.ClickException(
                "No active project in the current directory. Please specify the PID."
            )

    try:
        estela_client.get_project(pid)
    except Exception as ex:
        raise click.ClickException(
            "The project does not exist, or you do not have permission to perform this action."
        )

    spiders = estela_client.get_spiders(pid)
    spiders = [[spider["name"], str(spider["sid"])] for spider in spiders]

    headers = ["NAME", "SID"]
    click.echo(tabulate(spiders, headers, numalign="left", tablefmt="plain"))



import os
import yaml
import csv
import json
import click

from datetime import datetime
from estela_cli.templates import (
    ESTELA_AUTH_NAME,
    ESTELA_YAML_NAME,
    ESTELA_DIR,
    DATA_DIR,
    DOCKERFILE_NAME,
)


def get_project_path():
    return os.path.abspath(".")


def get_home_path():
    return os.path.expanduser("~")


def get_estela_yaml_path():
    project_path = get_project_path()
    return os.path.join(project_path, ESTELA_YAML_NAME)


def get_estela_dockerfile_path():
    project_path = get_project_path()
    return os.path.join(project_path, ESTELA_DIR, DOCKERFILE_NAME)


def get_host_from_env():
    return os.environ.get("ESTELA_API_HOST")


def get_username_from_env():
    return os.environ.get("ESTELA_USERNAME")


def get_password_from_env():
    return os.environ.get("ESTELA_PASSWORD")


def get_estela_settings():
    estela_yaml_path = get_estela_yaml_path()

    assert os.path.exists(estela_yaml_path), "{} not found.".format(ESTELA_YAML_NAME)

    with open(estela_yaml_path, "r") as estela_yaml:
        estela_config = yaml.full_load(estela_yaml)

    return estela_config


def get_estela_auth():
    home_path = get_home_path()
    estela_auth_path = os.path.join(home_path, ESTELA_AUTH_NAME)

    if not os.path.exists(estela_auth_path):
        return None

    with open(estela_auth_path, "r") as estela_auth_yaml:
        estela_auth = yaml.full_load(estela_auth_yaml)

    return estela_auth


def format_time(date):
    date = datetime.strptime(date, "%Y-%m-%dT%H:%M:%S.%fZ")
    return date.strftime("%Y-%m-%d %H:%M")


def format_key_value_pairs(key_value_pairs):
    if not key_value_pairs:
        return ""

    result = ""
    for key_value in key_value_pairs[:-1]:
        result += "{}: {}\n".format(key_value["name"], key_value["value"])

    result += "{}: {}".format(key_value_pairs[-1]["name"], key_value_pairs[-1]["value"])
    return result


def format_tags(tags):
    if not tags:
        return ""
    result = ""
    for tag in tags[:-1]:
        result += "{}\n".format(tag["name"])
    result += "{}".format(tags[-1]["name"])
    return result


def save_chunk_data(filename, data):
    project_path = get_project_path()
    filename = os.path.join(project_path, DATA_DIR, filename)
    if not os.path.exists(DATA_DIR):
        os.makedirs(DATA_DIR)
    if not os.path.exists(filename):
        with open(filename, "w", encoding="utf-8") as F:
            F.write("[")
    with open(filename, "a", encoding="utf-8") as F:
        for item in data:
            F.write(json.dumps(item))
            F.write(",")


def save_data(filename, tmp_filename, format):
    project_path = get_project_path()
    filename = os.path.join(project_path, DATA_DIR, filename)
    tmp_filename = os.path.join(project_path, DATA_DIR, tmp_filename)
    with open(tmp_filename, "rb+") as F:
        F.seek(-1, 2)
        F.truncate()
        F.write(b"]\n")

    if format == "json":
        os.rename(tmp_filename, filename)
    elif format == "csv" or format == "tsv":
        delimiter = "," if format == "csv" else "\t"
        with open(tmp_filename, "r", encoding="utf-8") as F:
            data = json.load(F)
        with open(filename, "w", encoding="utf-8") as F:
            keys = data[0].keys()
            writer = csv.DictWriter(F, delimiter=delimiter, fieldnames=keys)
            writer.writeheader()
            for row in data:
                writer.writerow({k: v for k, v in row.items() if k in keys})
            os.remove(tmp_filename)


def validate_key_value_format(ctx, param, value):
    try:
        key_value_pairs = []
        for pair in value:
            key, value = pair.split("=", 1)
            key_value_pairs.append({"name": key, "value": value})
        return key_value_pairs
    except:
        raise click.BadParameter("format must be 'NAME=VALUE'")


def validate_positive(ctx, param, value):
    if value is not None and value <= 0:
        raise click.BadParameter("must be a positive integer.")
    return value


def set_tag_format(ctx, param, value):
    tags = []
    for tag_name in value:
        tags.append({"name": tag_name})
    return tags


def _in(path, ignore_files):
    for file in ignore_files:
        if path.startswith(file):
            return True



import click

from estela_cli.login import login
from estela_cli.utils import get_estela_settings

SHORT_HELP = "Stop an active job"


@click.command(short_help=SHORT_HELP)
@click.argument("jid", required=True)
@click.argument("sid", required=True)
@click.argument("pid", required=False)
def estela_command(jid, sid, pid):
    """Stop an active job

    \b
    JID is the job's jid
    SID is the spider's sid
    PID is the project's pid (active project by default)
    """

    estela_client = login()
    if pid is None:
        try:
            estela_settings = get_estela_settings()
            pid = estela_settings["project"]["pid"]
        except:
            raise click.ClickException(
                "No active project in the current directory. Please specify the PID."
            )
    try:
        response = estela_client.stop_spider_job(pid, sid, jid)
        click.echo(f"job/spider-job-{jid}-{pid} stopped.")
    except Exception as ex:
        raise click.ClickException(
            "The job is not active, does not exist, or you do not have permission to perform this action."
        )



import os
import click

from string import Template
from estela_cli.login import login
from estela_cli.utils import (
    get_project_path,
    get_estela_yaml_path,
    get_estela_dockerfile_path,
)
from estela_cli.templates import (
    DATA_DIR,
    DOCKER_DEFAULT_ENTRYPOINT,
    DOCKER_REQUESTS_ENTRYPOINT,
    DOCKERFILE,
    DOCKERFILE_NAME,
    ESTELA_YAML,
    ESTELA_YAML_NAME,
    DOCKER_DEFAULT_REQUIREMENTS,
    DOCKER_DEFAULT_PYTHON_VERSION,
    ESTELA_DIR,
)

ALLOWED_PLATFORMS = ["scrapy", "requests"]
SHORT_HELP = "Initialize estela project for existing web scraping project"


def gen_estela_yaml(estela_client, entrypoint_path, pid=None):
    estela_yaml_path = get_estela_yaml_path()

    if os.path.exists(estela_yaml_path):
        raise click.ClickException("{} file already exists.".format(ESTELA_YAML_NAME))

    try:
        if pid is None:
            pid = click.prompt("Project ID")

        project = estela_client.get_project(pid)
    except:
        raise click.ClickException(
            "The project with id '{}' does not exists.".format(pid)
        )

    template = Template(ESTELA_YAML)
    values = {
        "project_pid": pid,
        "project_data_path": DATA_DIR,
        "python_version": DOCKER_DEFAULT_PYTHON_VERSION,
        "requirements_path": DOCKER_DEFAULT_REQUIREMENTS,
        "entrypoint": entrypoint_path,
    }

    result = template.substitute(values)

    with open(estela_yaml_path, "w") as estela_yaml:
        estela_yaml.write(result)
        click.echo("{} file created successfully.".format(ESTELA_YAML_NAME))


def gen_dockerfile(requirements_path, entrypoint_path):
    dockerfile_path = get_estela_dockerfile_path()

    if os.path.exists(dockerfile_path):
        raise click.ClickException(
            "{}/{} already exists.".format(ESTELA_DIR, DOCKERFILE_NAME)
        )

    project_path = get_project_path()
    requirements_local_path = os.path.join(project_path, requirements_path)
    if not os.path.exists(requirements_local_path):
        with open(requirements_local_path, "w") as requirementes:
            pass

    template = Template(DOCKERFILE)
    values = {
        "python_version": DOCKER_DEFAULT_PYTHON_VERSION,
        "requirements_path": requirements_path,
        "entrypoint": entrypoint_path,
    }
    result = template.substitute(values)

    with open(dockerfile_path, "w") as dockerfile:
        dockerfile.write(result)
        click.echo("{}/{} created successfully.".format(ESTELA_DIR, DOCKERFILE_NAME))


@click.command(short_help=SHORT_HELP)
@click.argument("pid", required=True)
@click.option(
    "-p",
    "--platform",
    type=click.Choice(ALLOWED_PLATFORMS, case_sensitive=False),
    default="scrapy",
    help="Platform to use, it can be 'scrapy' or 'requests'",
    show_default=True,
)
@click.option(
    "-r",
    "--requirements",
    default=DOCKER_DEFAULT_REQUIREMENTS,
    help="Relative path to requirements inside your project",
    show_default=True,
)
def estela_command(pid, platform, requirements):
    """Initialize estela project

    PID is the project's pid
    """
    platform_map = {
        "scrapy": DOCKER_DEFAULT_ENTRYPOINT,
        "requests": DOCKER_REQUESTS_ENTRYPOINT,
    }
    if not os.path.exists(ESTELA_DIR):
        os.makedirs(ESTELA_DIR)
    estela_client = login()
    try:
        response = estela_client.update_project(pid, framework=platform.upper(), action="update")
    except Exception as e:
        raise click.ClickException("Could not update framework project: %s" % str(e))
    finally:
        click.echo(f"{pid} is initialized as a {platform.capitalize()} project.")
    gen_estela_yaml(estela_client, platform_map[platform], pid)
    gen_dockerfile(requirements, platform_map[platform])



from setuptools import setup, find_packages

setup(
    name="estela",
    version="0.2.4",
    description="Estela Command Line Interface",
    packages=find_packages(),
    entry_points={
        "console_scripts": [
            "estela = estela_cli.__main__:cli",
        ],
    },
    install_requires=[
        "requests",
        "docker",
        "pyyaml",
        "click",
        "tabulate",
    ],
    classifiers=[
        "Development Status :: 3 - Alpha",
        "Operating System :: OS Independent",
        "Programming Language :: Python :: 3",
        "Topic :: Utilities",
    ],
)



import importlib

import click

from estela_cli import __version__

CONTEXT_SETTINGS = {"help_option_names": ["-h", "--help"]}


@click.group(context_settings=CONTEXT_SETTINGS)
@click.version_option(__version__)
def cli():
    pass


commands = [
    "list",
    "create",
    "update",
    "delete",
    "stop",
    "deploy",
    "login",
    "logout",
    "init",
    "context",
    "data",
]

for command in commands:
    module = importlib.import_module("estela_cli.{}".format(command))
    cli.add_command(module.estela_command, command)



import os
import click

from getpass import getpass
from string import Template
from estela_cli.estela_client import EstelaClient
from estela_cli.utils import (
    get_host_from_env,
    get_username_from_env,
    get_password_from_env,
    get_estela_auth,
    get_home_path,
)
from estela_cli.templates import ESTELA_AUTH_NAME, ESTELA_AUTH


DEFAULT_ESTELA_API_HOST = "http://localhost"


SHORT_HELP = "Save your credentials"


def env_login():
    host = get_host_from_env()
    username = get_username_from_env()
    password = get_password_from_env()

    try:
        estela_client = EstelaClient(host, username, password)
    except:
        raise Exception("Unable to login with environment credentials.")

    return estela_client


def yaml_login():
    estela_auth = get_estela_auth()

    if estela_auth is None:
        raise Exception(
            "File ~/{} not found. It is recommended to login with 'estela login'.".format(
                ESTELA_AUTH_NAME
            )
        )

    try:
        estela_client = EstelaClient(estela_auth["host"], token=estela_auth["token"])
    except:
        raise Exception(
            "Invalid context stored in ~/{}. Please login again.".format(
                ESTELA_AUTH_NAME
            )
        )

    return estela_client


def prompt_login(username=None, password=None, host=None):
    try:
        if host is None:
            host = click.prompt("Host", default=DEFAULT_ESTELA_API_HOST)
        if username is None:
            username = click.prompt("Username")
        if password is None:
            password = getpass()

        estela_client = EstelaClient(host, username, password)
    except:
        raise Exception("Unable to login with provided credentials.")

    return estela_client


def login(username=None, password=None, host=None):
    try:
        estela_client = env_login()
        click.echo(
            "You are currently using environment variables to configure estela CLI."
        )
        return estela_client
    except:
        pass

    try:
        estela_client = yaml_login()
        return estela_client
    except Exception as ex:
        click.echo(str(ex))

    try:
        estela_client = prompt_login(username, password, host)
    except Exception as ex:
        raise click.ClickException(str(ex))

    return estela_client


@click.command(short_help=SHORT_HELP)
@click.option(
    "--username",
    help="Username for login. If username is not given, it will be asked",
)
@click.option(
    "--password",
    help="Password for login. If password is not given, it will be asked",
)
@click.option(
    "--host",
    help="API endpoint to send the requests. If host is not given, it will be asked",
)
def estela_command(username, password, host):
    home_path = get_home_path()
    estela_auth_path = os.path.join(home_path, ESTELA_AUTH_NAME)

    if os.path.exists(estela_auth_path):
        raise click.ClickException(
            "You already logged in. To change credentials run 'estela logout' first."
        )

    try:
        estela_client = prompt_login(username, password, host)
    except Exception as ex:
        raise click.ClickException(str(ex))

    template = Template(ESTELA_AUTH)
    values = {
        "estela_host": estela_client.host,
        "estela_token": estela_client.token,
    }
    result = template.substitute(values)

    with open(estela_auth_path, "w") as estela_auth_yaml:
        estela_auth_yaml.write(result)
        click.echo(
            "Successful login. API Token stored in ~/{}.".format(ESTELA_AUTH_NAME)
        )



import click
import requests

from estela_cli.login import env_login, yaml_login
from estela_cli.utils import (
    get_estela_auth,
    get_estela_settings,
    get_host_from_env,
    get_username_from_env,
    get_password_from_env,
)
from estela_cli.templates import ESTELA_AUTH_NAME, OK_EMOJI, BAD_EMOJI


SHORT_HELP = "Show your current context"


OK_HOST = OK_EMOJI + " Host: {}."
BAD_HOST = "{} Cannot connect to the Host.".format(BAD_EMOJI)

OK_AUTH = "{} Valid Auth.".format(OK_EMOJI)
BAD_AUTH = "{} Invalid Auth.".format(BAD_EMOJI)

NO_PROJECT = "No active project in the current directory."
ACTIVE_PROJECT = "Active project: {}."


def test_host(host):
    response = requests.get("{}/api".format(host), timeout=1)
    assert "projects" in response.json()


@click.command(short_help=SHORT_HELP)
def estela_command():
    host = get_host_from_env()
    username = get_username_from_env()
    password = get_password_from_env()

    if host is None or username is None or password is None:
        estela_auth = get_estela_auth()

        if estela_auth is None:
            click.echo(
                "Environment variables not declared, ~/{} not found.".format(
                    ESTELA_AUTH_NAME
                )
            )
            return

        click.echo(
            "You are currently using ~/{} file to configure estela CLI.".format(
                ESTELA_AUTH_NAME
            )
        )

        try:
            test_host(estela_auth["host"])
            click.echo(OK_HOST.format(estela_auth["host"]))
        except:
            click.echo(BAD_HOST)
            return
        try:
            estela_client = yaml_login()
            click.echo(OK_AUTH)
        except:
            click.echo(BAD_AUTH)
            return
    else:
        click.echo(
            "You are currently using environment variables to configure estela CLI."
        )
        try:
            test_host(host)
            click.echo(OK_HOST.format(host))
        except:
            click.echo(BAD_HOST)
            return
        try:
            estela_client = env_login()
            click.echo(OK_AUTH)
        except:
            click.echo(BAD_AUTH)
            return

    try:
        settings = get_estela_settings()
        click.echo(ACTIVE_PROJECT.format(settings["project"]["pid"]))
    except:
        click.echo(NO_PROJECT)



import click
import os

from estela_cli.utils import get_home_path
from estela_cli.login import ESTELA_AUTH_NAME


SHORT_HELP = "Remove your credentials"


@click.command(short_help=SHORT_HELP)
def estela_command():
    home_path = get_home_path()
    estela_auth_path = os.path.join(home_path, ESTELA_AUTH_NAME)

    if not os.path.exists(estela_auth_path):
        raise click.ClickException("You have not logged in. Run 'estela login' first.")

    os.remove(estela_auth_path)
    click.echo("Successful logout.")
